char *helpstring=""
"Easy Monte Carlo Code (emc2)\n"
"by Axel van de Walle\n"
"\n"
"Command line parameters:\n"
"\n"
"-T0 : initial temperature\n"
"-T1 : final temperature\n"
"-dT : temperature step\n"
"-db : inverse temperature step\n"
"\n"
"Temperatures are given in units of energy unless the -k option is set.\n"
"-dT and -db are mutually exclusive. Which option you use determines\n"
"whether T or 1/T are unformly spaced. The output file always shows T.\n"
"\n"
"-k : Sets boltman'z constant (default k=1). This only affects how\n"
"     temperatures are converted in energies.  -k=8.617e-5 lets you enter\n"
"     temperatures in kelvins when energies are in eV.\n"
"     (You can also select this value by using the -keV option.)\n"
"\n"
"-mu0 : initial chemical potential\n"
"-mu1 : final chemical potential\n"
"\n"
"  By default, these are input as dimensionless values defined as\n"
"  follows: If x is integer, x is the chemical potential that\n"
"  stabilizes a two phase equilibrium between ground state x-1 and\n"
"  x. (Ground states are numbered starting at 0.) Fractional values\n"
"  interpolate linearly between these integer values.  Negative values\n"
"  or values larger than the number of ground states - 1 are allowed.\n"
"  The values of mu are found by linear extrapolation.  If the -abs\n"
"  option is specified, the value of mu are in units of energy per spin\n"
"  value (as in the output file).  (The ground states are read in from\n"
"  the file gs_str.out .)\n"
"\n"
"  If there are only two ground states, the only correction performed is\n"
"  to shift mu so that mu=0 stabilizes a two-phase equilibrium between\n"
"  the two ground states.\n"
"  If there are less than 2 ground states (!), no correction is made.\n"
"\n"
"-dmu : chemical potential step (expressed in the same units as mu0 and mu1).\n"
"\n"
"NOTE: If mu1 is omitted, only a scan through T is performed keeping mu=mu0.\n"
"      If T1 is omitted, only scan through mu is performed keeping T=T0.\n"
"\n"
"-gs : Gives the index of the ground state to use as an initial spin\n"
"      configuration (the leftmost pure element is numbered 0).  If the\n"
"      index is -1, the disordered state with an average spin of zero\n"
"      is used as the starting configuration.\n"
"\n"
"-phi0 : value of the grand canonical potential at the initial point of\n"
"        the run.  If left unspecified, it is set to the grand canonical\n"
"        potential of given by either the 1-spin Low Temperature Expansion\n"
"        or the High Temperature Expansion, depending whether the initial\n"
"        state is an ordered or a disordered phase.\n"
"\n"
"-er : enclosed radius. The Monte Carlo cell will the smallest possible\n"
"    supercell of unit cell of the initial configuration such that a\n"
"    sphere of that radius fits inside it.  This allows you to specify\n"
"    the system size in a structure-independent fashion.\n"
"\n"
"-innerT : by default, the inner loop is over values mu while the outer\n"
"          loop is over values of T. This flag permutes that order.\n"
"\n"
"-eq : number of equilibration passes. (Equilibration is performed at\n"
"      the beginning of each step of the outer loop.)\n"
"\n"
"-n : number of Monte Carlo passes performed to evaluate thermodynamic\n"
"     quanties at each step of the inner loop.\n"
"\n"
"-dx: instead of specifying -eq and -n, you can ask the code to equilibrate\n"
"     and run for a time such that the average concentration is accurate\n"
"     within the target precision specifified by -dx=.\n"
"\n"
"-tstat : Critical value of the test for discontinuity. The code is\n"
"         able to catch phase transformations (most of the time) when\n"
"         going from an ordered phase to another or to the disordered\n"
"         state. This involves a statistical test which a\n"
"         user-specified confidence level. The default, 3, corresponds\n"
"         to a 0.4% chance of finding a transition where there is\n"
"         none. (Refer to a standard normal table.)  -tstat=0 turns off\n"
"         this option.  Suggestion: if a phase transition is\n"
"         undetected, first try to reduce dT or dmu or increase n or decrease dx\n"
"         before toying around with this parameter.  Also: beware of\n"
"         hysteresis.\n"
"\n"
"-sigdig : Number of significant digits printed. Default is 6.\n"
"\n"
"-o:       Name of the output file (default: mc.out).\n"
"\n"
"Tricks:\n"
"\n"
"   To read parameters from a file, use:\n"
"    emc2 `cat inputfile`\n"
"   where inputfile contains the commands line options.\n"
"\n"
"   To selectively display a few of the output quantities, use:\n"
"    emc2 [options] | cut -f n1,n2,n3,...\n"
"   where n1,n2,n3,... are the column number desired (see below).\n"
"\n"
"Input files:\n"
"\n"
"lat.in       : description of the lattice.\n"
"clusters.out : describes the clusters. \n"
"eci.out      : provides the ECI. \n"
"gs_str.out   : a list of ground states, in increasing order of concentration.\n"
"\n"
"   These 4 files can be created by maps.\n"
"   See maps documentation (maps -h) for a description of the formats.\n"
"\n"
"Optional input file:\n"
"\n"
"teci.out     : if present, provides temperature-dependent eci (overrides eci.out)\n"
"               (Note that, even when the teci.out file is used, column 6 of the output file\n"
"                reflects only the configurational contribution to the heat capacity.)\n"
"\n"
"The format this file is:\n"
"[maximum temperature: Tm]\n"
"[number of temperatures where eci are provided: nT]\n"
"[ECIs for temperature 0]\n"
"[ECIs for temperature Tm/(nT-1)]\n"
"[ECIs for temperature 2*Tm/(nT-1)]\n"
"...\n"
"[ECIs for temperature Tm]\n"
"\n"
"Note that these numbers can be seperated by blanks or newlines, as desired.\n"
"\n"
"Format of the output file:\n"
"\n"
"Each line contains, in order:\n"
"\n"
" 1) T:  Temperature\n"
" 2) mu: chemical potential (derivative of the free energy wrt to average spin)\n"
" 3) E-mu*x:  Average energy (per spin)\n"
" 4) x:  Average Concentration [ranges from -1 to 1]\n"
" 5) phi: grand canonical potential\n"
" 6) E2: Variance of the energy (proportional to heat capacity)\n"
" 7) x2: Variance of the concentration (proportional to susceptibility)\n"
" 8) E_lte-mu*x_lte: calculated with a one spin Low Temperature Expansion\n"
" 9) x_lte\n"
"10) phi_lte\n"
"11) E_mf-mu*x_mf:  calculated with the Mean Field approximation\n"
"12) x_mf\n"
"13) phi_mf\n"
"14) E_hte-mu*x_hte: calculated with a High Temperature Expansion (ideal solution + polynomial in x)\n"
"15) x_hte\n"
"16) phi_hte\n"
"17) lro: Long Range Order parameter of the initial ordered phase (=0 if initial phase is disordered).\n"
"18-) corr: the average correlations associated with each cluster.\n"
"\n"
"NOTE: to obtain 'canonical' rather than grand canonical quantities,\n"
"use the -g2c option. This has the effect of adding mu*x to columns\n"
"3,5,8,10,11,13,14,16.\n";
